/*
 * AVR based RGB Moodlight
 *
 * moodlight.c - moodlight modes and remote control
 *
 * (c) Axel (XL) Schwenke, axel.schwenke@gmx.net
 *
 * $Id: moodlight.c 152 2012-03-08 08:32:46Z schwenke $
 */

#include <inttypes.h>

#include "irmp.h"
#include "moodlight.h"


/* remote control address and command codes */

#define MY_PROTO    IRMP_NEC_PROTOCOL   /* IR protocol and */
#define MY_REMOTE   0x916e              /* system code */

#define KEY_OPERATE 0x14                /* on/off */
#define KEY_1       0x05                /* mode 1 */
#define KEY_2       0x06                /* mode 2 */
#define KEY_3       0x07                /* mode 3 */
#define KEY_4       0x0c
#define KEY_5       0x0d
#define KEY_6       0x0e
#define KEY_7       0x0f
#define KEY_8       0x1c
#define KEY_9       0x1d
#define KEY_0       0x04
#define KEY_MINUS   0x18
#define KEY_PLUS    0x19
#define KEY_REV     0x02                /* slower */
#define KEY_FWD     0x03                /* faster */
#define KEY_STOP    0x01
#define KEY_PLAY    0x08                /* change direction */
#define KEY_REC     0x09
#define KEY_PAUSE   0x0b                /* stop/start */


/* bits in ml.flags */

#define ML_FLAG_OFF   0x80
#define ML_FLAG_PAUSE 0x02
#define ML_FLAG_UP    0x01

/* more moodlight constants */

#define ML_MODE3_PERIOD 0x40  /* must be a power of 2 */

/* global moodlight variables(s) */

struct {
    uint8_t  mode;      /* the active moodlight mode */
    uint8_t  delay;     /* run moodlight_step() every "delay" PWM cycles */
    uint16_t cycle;     /* counter used by modes */
    uint8_t  flags;     /* on/off, stopped, direction */
} ml;

/* led brightness (0..MAX_BRIGHT) */

uint8_t level_r;
uint8_t level_g;
uint8_t level_b;

/* clock tick, decremented once in each PWM cycle */

volatile uint8_t tick;

/* last received IR signal */

IRMP_DATA irmp_data;


/* evaluate an IR remote signal */
void moodlight_remote(void)
{
    if (   irmp_data.protocol != MY_PROTO
        || irmp_data.address  != MY_REMOTE) {
        return;
    }

    switch (irmp_data.command) {

    case KEY_OPERATE: /* on/off */
        {
            if ((irmp_data.flags & IRMP_FLAG_REPETITION) == 0) {
                ml.flags ^= ML_FLAG_OFF;
                if (ml.flags & ML_FLAG_OFF) {
                    level_r = 0;
                    level_g = 0;
                    level_b = 0;
                } else {
                    moodlight_wakeup();
                }
            }
            break;
        }

    case KEY_1:
        {
            ml.mode = 1;
            ml.delay = 6;
            ml.flags = 0;
            ml.cycle = 1;
            tick = 1;
            break;
        }

    case KEY_2:
        {
            ml.mode = 2;
            ml.delay = 4;
            ml.flags = 0;
            ml.cycle = 1;
            tick = 1;
            break;
        }

    case KEY_3:
        {
            ml.mode = 3;
            ml.delay = 4;
            ml.flags = 0;
            ml.cycle = 0;
            tick = 1;
            break;
        }

    case KEY_FWD: /* faster */
        {
            if ((ml.mode == 1 || ml.mode == 2 || ml.mode == 3)
                && ml.delay > 1) {
                ml.delay--;
            }
            break;
        }

    case KEY_REV: /* slower */
        {
            if (ml.mode == 1 && ml.delay < 32) {
                ml.delay++;
            }
            if ((ml.mode == 2 || ml.mode == 3) && ml.delay < 20) {
                ml.delay++;
            }
            break;
        }

    case KEY_PAUSE: /* stop motion */
        {
            if ((irmp_data.flags & IRMP_FLAG_REPETITION) == 0) {
                ml.flags ^= ML_FLAG_PAUSE;
            }
            break;
        }

    case KEY_PLAY:  /* start motion / change direction */
        {
            if ((irmp_data.flags & IRMP_FLAG_REPETITION) == 0) {
                if (ml.flags & ML_FLAG_PAUSE) {
                    ml.flags ^= ML_FLAG_PAUSE;
                } else {
                    ml.flags ^= ML_FLAG_UP;
                }
            }
            break;
        }

    }
}


/* initialization */
void moodlight_init(void)
{
    ml.mode  = 1;
    ml.delay = 6;
    ml.flags = 0;
    ml.cycle = 1;
    tick = 1;
}


/* called after waking up from power-off */
void moodlight_wakeup(void)
{
    switch (ml.mode) {
    case 3:
        {
            ml.cycle &= ~(ML_MODE3_PERIOD-1);
            break;
        }
    }
    tick = 1;
}


/* called after last pwm phase when tick == 0 */
void moodlight_step(void)
{
    if (ml.flags & ML_FLAG_OFF) {
        return;
    }

    switch (ml.mode) {

    case 1: /* fading R-G-B */
        {
            if (ml.cycle <= MAX_BRIGHT) {
                level_r = (MAX_BRIGHT+1) - ml.cycle;
                level_g = ml.cycle;
                level_b = 0;
            }
            else if (ml.cycle <= 2*MAX_BRIGHT) {
                level_r = 0;
                level_g = (2*MAX_BRIGHT+1) - ml.cycle;
                level_b = ml.cycle - MAX_BRIGHT;
            }
            else {
                level_r = ml.cycle - (2*MAX_BRIGHT);
                level_g = 0;
                level_b = (3*MAX_BRIGHT+1) - ml.cycle;
            }

            if ((ml.flags & ML_FLAG_PAUSE) == 0) {
                if (ml.flags & ML_FLAG_UP) {
                    ml.cycle = (ml.cycle == 3*MAX_BRIGHT+1 ? 1 : ml.cycle+1);
                } else {
                    ml.cycle = (ml.cycle == 0 ? 3*MAX_BRIGHT : ml.cycle-1);;
                }
            }
            tick = ml.delay;
            break;
        }

    case 2: /* fading R-R+G-G-G+B-B-B+R */
        {
            if (ml.cycle <= MAX_BRIGHT) {
                /* R -> R+G */
                level_r = MAX_BRIGHT;
                level_g = ml.cycle;
                level_b = 0;
            }
            else if (ml.cycle <= 2*MAX_BRIGHT) {
                /* R+G -> G */
                level_r = 2*MAX_BRIGHT+1-ml.cycle;
                level_g = MAX_BRIGHT;
                level_b = 0;
            }
            else if (ml.cycle <= 3*MAX_BRIGHT) {
                /* G -> G+B */
                level_r = 0;
                level_g = MAX_BRIGHT;
                level_b = ml.cycle - 2*MAX_BRIGHT;
            }
            else if (ml.cycle <= 4*MAX_BRIGHT) {
                /* G+B -> B */
                level_r = 0;
                level_g = 4*MAX_BRIGHT+1 - ml.cycle;
                level_b = MAX_BRIGHT;
            }
            else if (ml.cycle <= 5*MAX_BRIGHT) {
                /* B -> B+R */
                level_r = ml.cycle - 4*MAX_BRIGHT;
                level_g = 0;
                level_b = MAX_BRIGHT;
            }
            else {
                /* B+R -> R */
                level_r = MAX_BRIGHT;
                level_g = 0;
                level_b = 6*MAX_BRIGHT+1 - ml.cycle;
            }

            if ((ml.flags & ML_FLAG_PAUSE) == 0) {
                if (ml.flags & ML_FLAG_UP) {
                    ml.cycle = (ml.cycle == 6*MAX_BRIGHT+1 ? 1 : ml.cycle+1);
                } else {
                    ml.cycle = (ml.cycle == 0 ? 6*MAX_BRIGHT : ml.cycle-1);;
                }
            }
            tick = ml.delay;
            break;
        }

    case 3: /* switching R-R+G-G-G+B-B-R+B-R+G+B */
        {
            if ((ml.cycle & (ML_MODE3_PERIOD-1)) == 0) {
                switch (ml.cycle/ML_MODE3_PERIOD) {
                case 0:
                    {
                        level_r = MAX_BRIGHT;
                        level_g = 0;
                        level_b = 0;
                        break;
                    }
                case 1:
                    {
                        level_r = MAX_BRIGHT;
                        level_g = MAX_BRIGHT;
                        level_b = 0;
                        break;
                    }
                case 2:
                    {
                        level_r = 0;
                        level_g = MAX_BRIGHT;
                        level_b = 0;
                        break;
                    }
                case 3:
                    {
                        level_r = 0;
                        level_g = MAX_BRIGHT;
                        level_b = MAX_BRIGHT;
                        break;
                    }
                case 4:
                    {
                        level_r = 0;
                        level_g = 0;
                        level_b = MAX_BRIGHT;
                        break;
                    }
                case 5:
                    {
                        level_r = MAX_BRIGHT;
                        level_g = 0;
                        level_b = MAX_BRIGHT;
                        break;
                    }
                case 6:
                    {
                        level_r = MAX_BRIGHT;
                        level_g = MAX_BRIGHT;
                        level_b = MAX_BRIGHT;
                        break;
                    }
                }
            }

            if ((ml.flags & ML_FLAG_PAUSE) == 0) {
                if (ml.flags & ML_FLAG_UP) {
                    ml.cycle = (ml.cycle == (7*ML_MODE3_PERIOD-1) ? 0 : ml.cycle+1);
                } else {
                    ml.cycle = (ml.cycle == 0 ? (7*ML_MODE3_PERIOD-1) : ml.cycle-1);;
                }
            }
            tick = ml.delay;
            break;
        }

    }
}

